<?php
/* --------------------------------------------------------------
   Pager.inc.php 2018-07-02
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2018 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/

/**
 * Pager
 */
class Pager
{
	/**
	 * @var int
	 */
	private $page;
	
	/**
	 * @var int
	 */
	private $perPage;
	
	/**
	 * @var int
	 */
	private $offset;
	
	
	/**
	 * Pager constructor.
	 *
	 * @param IntType $page    Current page.
	 * @param IntType $perPage Items per page.
	 */
	public function __construct(IntType $page, IntType $perPage)
	{
		$this->page    = $page->asInt();
		$this->perPage = $perPage->asInt();
		$this->offset  = $this->page <= 1 ? 0 : ($this->page - 1) * $this->perPage;
	}
	
	
	/**
	 * Named constructor of pager.
	 *
	 * @param int $page    Current page.
	 * @param int $perPage Items per page.
	 *
	 * @return Pager New instance.
	 */
	public static function create($page, $perPage)
	{
		return MainFactory::create(static::class, new IntType($page), new IntType($perPage));
	}
	
	
	/**
	 * Returns the current page.
	 *
	 * @return int
	 */
	public function page()
	{
		return $this->page;
	}
	
	
	/**
	 * Returns the items count per page.
	 *
	 * @return int
	 */
	public function perPage()
	{
		return $this->perPage;
	}
	
	
	/**
	 * Returns the offset.
	 *
	 * @return int
	 */
	public function offset()
	{
		return $this->offset;
	}
}